<?php
/***************************************************************************
 *   
 *   countryiptables.php
 *
 *   copyright (C) 2006-2011 Click Here Software Co.
 *
 *   written by: Maurice Randall
 *
 *   distributed by: FixingTheWeb.com
 *
 *   email: support@fixingtheweb.com
 *
 *   Version 1.6beta : July 18, 2011
 *
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is used for selecting the countries you wish to block from
 *   having access to your server. It will then create a file which can be
 *   inserted into the Linux kernel and used by iptables.
 *
 *   Before using this program, you must run the setup utility countryipsetup.php.
 *   Also, be sure to read the documentation that was included with this package.
 *
 ***************************************************************************/

/***************************************************************************
 *
 *   This product can use either one of the following two country database
 *   files:
 *
 *   The IP-to-Country Database
 *   provided by WebHosting.Info (http://www.webhosting.info),
 *   available from http://ip-to-country.webhosting.info
 *
 *   or:
 *
 *   The GeoIP Database
 *   provided by MaxMind.com (http://www.maxmind.com),
 *   available from http://www.maxmind.com
 *
 ***************************************************************************/

/***************************************************************************
 *   
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *   
 ***************************************************************************/

// The version number of this release.
$COUNTRY_VERSION="V1.6beta";

// The name of the file containing the country list.
$COUNTRY_LIST="countries.php";

// Amount of time (in seconds) given to the server to create the file.
$DEF_TIME_LIMIT=600;

// The default filename to use for creating the iptables list.
$DEF_IPT_FILENAME="badiptlist.txt";

// The name of the file containing the default iptables rules.
$DEF_IPT_RULES="defaultrules";

// Check if the countries.php file is available.
// If so, read it in and sort the list.
if (file_exists($COUNTRY_LIST)) {
	include($COUNTRY_LIST);
} else {
	errorExit("Database file not found: ($COUNTRY_LIST)");
}

if (isset($iptfilename)) {
	$iptablesfilename=$iptfilename;
} else {
	$iptablesfilename=$DEF_IPT_FILENAME;
}

if (isset($ethDefault)) {
	$eth=$ethDefault;
} else {
	$eth="eth0";
}

if (isset($ethOtherDefault)) {
	$ethOther=$ethOtherDefault;
} else {
	$ethOther="eth3";
}

if (isset($ethmethodDefault)) {
	$ethmethod=$ethmethodDefault;
} else {
	$ethmethod=0;
}

if ($ethmethod<>1) {
	$ethmethod=0;
}

if ((isset($_POST['SaveDefaults'])) ||
	(isset($_POST['CreateFile'])) ||
	(isset($_POST['ClearAll'])) ||
	(isset($_POST['SelectAll'])) ||
	(isset($_POST['DisplayResults']))) {
	if (isset($_POST['iptablesfilename'])) {
		$iptablesfilename=$_POST['iptablesfilename'];
	}
	if (isset($_POST['eth'])) {
		$eth=$_POST['eth'];
	}
	if (isset($_POST['ethOther'])) {
		$ethOther=$_POST['ethOther'];
	}
	if (isset($_POST['ethmethod'])) {
		$ethmethod=1;
	} else {
		$ethmethod=0;
	}
}

if ($iptablesfilename=="") {
	$iptablesfilename=$DEF_IPT_FILENAME;
}

$kernels = "newer";
if (isset($_POST['kernels'])) { $kernels = $_POST['kernels']; }

$bad_countries = array();


if (isset($_POST['SaveDefaults']) ||
	isset($_POST['CreateFile']) ||
	isset($_POST['DisplayResults']) )
{
	$cs = $_POST['cs'];
	$n = count($cs);
	foreach($countries as $key => $value) {
	$countries[$key][0]="OK";
		$i = 0;
		while ($i < $n) {
			if ($key==$cs[$i]) {
				$countries[$key][0]="BAD";
				$bad_countries[] = $key;
				break;
			}
			$i++;
		}
	}
}

if (isset($_POST['SaveDefaults'])) {
	if (is_writeable("./")==FALSE) {
		errorExit("Unable to write in this directory");
	}
	if (is_writeable($COUNTRY_LIST)) {
		if (($handle = fopen($COUNTRY_LIST, "w"))==FALSE) {
			errorExit("Unable to update file: $COUNTRY_LIST");
		}
	} else {
		errorExit("Unable to update file: $COUNTRY_LIST");
	}
	writeFileLine($handle, "<?php \n");
	writeFileLine($handle, "\$iptfilename=\"".$iptablesfilename."\";\n");
	writeFileLine($handle, "\$ethDefault=\"".$eth."\";\n");
	writeFileLine($handle, "\$ethOtherDefault=\"".$ethOther."\";\n");
	writeFileLine($handle, "\$ethmethodDefault=".$ethmethod.";\n");
	writeFileLine($handle, "\$countries=array(\n");
	foreach($countries as $key => $value) {
		if ($countries[$key][0]<>"BAD") {
			$countries[$key][0]="OK";
		}
		$key0=$countries[$key][0];
		$key1=$countries[$key][1];
		$fileLine="\"$key\" => array(\"$key0\",\"$key1\"),\n";
		writeFileLine($handle, $fileLine);
	}
	writeFileLine($handle, ");\n?>\n");
	fclose($handle);
}

if (isset($_POST['ClearAll'])) {
	foreach($countries as $key => $value) {
		$countries[$key][0]="OK";
	}
}

if (isset($_POST['SelectAll'])) {
	foreach($countries as $key => $value) {
		$countries[$key][0]="BAD";
	}
}

/*****************************************************************
 We execute this section if the "Create File" button is clicked.
*****************************************************************/

if ((isset($_POST['CreateFile'])) && ($iptablesfilename<>"")) {
	set_time_limit($DEF_TIME_LIMIT);
	$defaultStart="*nat\n";
	$defaultEnd="COMMIT\n";

	// Open the new or existing file for writing/rewriting.
	if (file_exists($iptablesfilename)) {
		if (is_writeable($iptablesfilename) ==FALSE) {
			errorExit("Unable to write to file: $iptablesfilename");
		}
	}
	if ((is_writeable("./"))==FALSE) {
		errorExit("Unable to write in this directory");
	} else {
		if (($handle = fopen($iptablesfilename, "w")) ==FALSE) {
			errorExit("Unable to create file: $iptablesfilename");
		}
	}

	// Add the beginning from the "defaultrules" file if it exists.
	$defRules=0;
	$endOfDefRules=1;
	if (file_exists($DEF_IPT_RULES)) {
		if (($defhandle = fopen($DEF_IPT_RULES, "r")) ==FALSE) {
			errorExit("Unable to read from file: $DEF_IPT_RULES");
		}
		$defRules=1;
		$endOfDefRules=0;
		while (true) {
			$defruleline = fgets($defhandle, 256);
			if ($defruleline=="") {
				$endOfDefRules=1;
				break;
			}
			if ($defruleline[0]<>"\x23") {
				$teststring=substr($defruleline, 0, 4 );
				if (($teststring=="*nat") || ($teststring=="*NAT")) {
					break;
				} else {
					if (fwrite($handle, $defruleline) == FALSE) {
						errorExit("Cannot write to file: ($iptablesfilename)");
					}
				}
			}
		}
	}

	if (fwrite($handle, $defaultStart) == FALSE) {
		exit;
	}

	if ($defRules) {
		if ($endOfDefRules==0) {
			$defruleline = fgets($defhandle, 256);
			if (fwrite($handle, $defruleline) == FALSE) {
				exit;
			}
			$defruleline = fgets($defhandle, 256);
			if (fwrite($handle, $defruleline) == FALSE) {
				exit;
			}
			$defruleline = fgets($defhandle, 256);
			if (fwrite($handle, $defruleline) == FALSE) {
				exit;
			}
		}
	}

	$rangeCount=0;

	do {

		if (file_exists($rangeCount.".php")) {
			include($rangeCount.".php");
		} else {
			errorExit("Database file not found: ($iptablesfilename)");
		}

		// Add content to the file.
		if ($eth=="other") {
			$ethBlock=$ethOther;
		} else {
			$ethBlock=$eth;
		}

		if ($ethmethod==1) {
			$icommand="-i ! ".$ethBlock;
		} else {
			$icommand="-i ".$ethBlock;
		}
		$oldRange="";
		$startFound=0;
		$iptext="-I PREROUTING $icommand -m iprange --src-range";

		foreach($ranges as $key => $value) {
			if (ckifbadcountry($ranges[$key][1])=="BAD") {
				$rangehigh=dectodot($ranges[$key][0]);
				if ($startFound==0) {
					$rangelow=dectodot($key);
					$startFound=1;
				}
			} else {
				if ($startFound) {
					$startFound=0;
					$range="$rangelow-$rangehigh";
					if ("$range"<>"$oldRange") {
						$oldRange="$range";
						if ($kernels == "newer") {
							$iptablesline="$iptext $range -j DNAT --to 0\n";
						} else {
							$iptablesline="$iptext $range -j DROP\n";
						}
					if (fwrite($handle, $iptablesline) == FALSE) {
						errorExit("Cannot write to file: ($iptablesfilename)");
					}
				}
			}
		}
	}

	if ($startFound) {
		$startFound=0;
		$range="$rangelow-$rangehigh";
		if ("$range"<>"$oldRange") {
			$oldRange="$range";
			if ($kernels == "newer") {
				$iptablesline="$iptext $range -j DNAT --to 0\n";
			} else {
				$iptablesline="$iptext $range -j DROP\n";
			}
			if (fwrite($handle, $iptablesline) == FALSE) {
				errorExit("Cannot write to file: ($iptablesfilename)");
			}
		}
	}

	$rangeCount+=1;

	} while ($rangeCount<256);


	if (($endOfDefRules) || ($defRules==0)) {
		if (fwrite($handle, $defaultEnd) == FALSE) {
			errorExit("Cannot write to file");
		}
	}

	if (($defRules) && ($endOfDefRules==0)) {
		while (true) {
			$defruleline = fgets($defhandle, 256);
			if ($defruleline=="") {
				break;
			}
			if ($defruleline[0]<>"\x23") {
				if (fwrite($handle, $defruleline) == FALSE) {
					errorExit("Cannot write to file");
				}
			}
		}
	}

	// Close the files.
	if ($defRules) {
		fclose($defhandle);
	}
	fclose($handle);

// End of the "Create File" section.
}

/***************************************************************
 Function section.
****************************************************************/


// This is called each time a line is written to the currently
// open file.
function writeFileLine($handle, $fileline) {
	if (fwrite($handle, $fileline) == FALSE) {
		errorExit("Cannot write to file");
	}
}

// Do a lookup to see if this is one of the countries to block.
function ckifbadcountry($ckcountry) {
	global $bad_countries;
	$countryck="OK";
	foreach($bad_countries as $value) {
		if ($ckcountry==$value) {
			$countryck="BAD"; break;
		}
	}
	return $countryck;
}

// Convert a 32 bit long decimal number to a 4 number dot notation.
function dectodot($decip) {
	$num0 = floor($decip/16777216);
	$tmp1 = $num0*16777216;
	$num1 = floor(($decip-$tmp1)/65536);
	$tmp2 = $num1*65536;
	$num2 = floor(($decip-$tmp1-$tmp2)/256);
	$tmp3 = $num2*256;
	$num3 = $decip-$tmp1-$tmp2-$tmp3;
	return $num0.".".$num1.".".$num2.".".$num3;
}


// This function is called when a severe enough error occurs and
// gives us a clean way out by finishing up the html file that is
// being generated and displayed to the user.
function errorExit($errorMsg) {
	print "</table>\n";
	print "</form>\n";
	print "<br><p><b>Error:</b> $errorMsg\n";
	print "\n</body>\n</html>\n";
	exit;
}

// End of the function section.

if (!isset($_POST['DisplayResults'])) {

/*********************************************************************
  Create the normal browser interface.
*********************************************************************/
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
<head>
<title>
IPTables Country Blocker
</title>
<meta http-equiv="Content-Type" content="text/html;charset=utf-8"/>
<style type="text/css">
.links{
 font-family:Tahoma, Geneva, sans-serif;
 font-size:13px;
}
</style>
</head>
<body>
<div class="links">
<font size="+1"><b>Generate Country Blocking Rules for IPTABLES</b></font>
<?php echo "<br>countryipblock package version: ".$COUNTRY_VERSION; ?>
<p/>Visit fixingtheweb.com for this and other useful stuff.
<p/>
<form method="post" action="menu.php">
<input type=submit  value="Main Menu" name="mainmenu"/>
</form>

<hr/>

<form method="post" action="countryiptables.php"> 
<table width="90%">
<tr>
<td valign="top">
<input type=submit  value="Create File" name="CreateFile"/>
<input type=submit value="Display In Browser" name="DisplayResults"/>

<p/>
<input type=submit value="Load Default Settings" name="LoadDefaults"/>
<p/>
<input type=submit value="Save As Defaults" name="SaveDefaults"/>

</td>
<td valign="top">

<b>SELECT INTERFACE TO BLOCK:</b>
<br/>
<input type="radio" name="eth" value="eth0" <?php if ($eth=="eth0") { print " checked"; } ?>/>eth0&nbsp;
<input type="radio" name="eth" value="eth1" <?php if ($eth=="eth1") { print " checked"; } ?>/>eth1&nbsp;
<input type="radio" name="eth" value="eth2" <?php if ($eth=="eth2") { print " checked"; } ?>/>eth2&nbsp;

<input type="radio" name="eth" value="other" <?php if ($eth=="other") { print " checked"; } ?>/>&nbsp;
<input type="text" name="ethOther" value=<?php print "\"$ethOther\""; ?>/>&nbsp;

<br/>
<input type="checkbox" name="ethmethod" <?php if ($ethmethod==1) { print " checked"; } ?>/>
All except above interface

<br/>
<input type="radio" name="kernels" value="newer" <?php if ($kernels=="newer") { print " checked=\"checked\""; } ?>/>newer kernels&nbsp;
<input type="radio" name="kernels" value="older" <?php if ($kernels=="older") { print " checked=\"checked\""; } ?>/>older kernels&nbsp;

<p/>
<b>FILENAME TO SAVE AS:</b>
<br/>
<input type="text" name="iptablesfilename" size="30" value="<?php print $iptablesfilename; ?>" />

</td>
</tr>
</table>
<hr/>
<b>SELECT THE COUNTRIES TO BLOCK:</b>

&nbsp;<input type=submit value="Select All" name="SelectAll"/>
&nbsp;<input type=submit value="Clear All" name="ClearAll"/>
<br/>&nbsp;

<?php

// Create the display of countries and checkboxes.

asort($countries);

print "<table width=\"96%\" align=\"center\">\n";

$cellCount=0;

foreach($countries as $key => $value) {
	if ($cellCount==0) {
		print "<tr>\n";
	}
	print "<td width=\"33%\" valign=\"top\">\n";
	$checked="";
	$curCountry=$countries[$key][1];
	if ($countries[$key][0]=="BAD") {
		$checked=" checked";
	}
	print "<input type=\"checkbox\" name=\"cs[]\" value=\"$key\" $checked>";
	print "$curCountry\n";
	print "</td>\n";
	$cellCount+=1;
	if ($cellCount==3) {
		print "</tr>\n"; $cellCount=0;
	}
}

if ($cellCount<>3) {
	while ($cellCount) {
		print "<td width=\"33%\" valign=\"top\">&nbsp;</td>\n";
		$cellCount-=1;
	}
	print "</tr>\n";
}

?>

</form>

</div>
</body>
</html>
<?php
} else {

include 'displayresults.php';

}

?>
